import { CoreStitchAuth, CoreStitchUser, StitchAppClientInfo, StitchCredential, StitchRequestClient, StitchUserFactory, Storage } from "mongodb-stitch-core-sdk";
import AuthProviderClientFactory from "../providers/internal/AuthProviderClientFactory";
import NamedAuthProviderClientFactory from "../providers/internal/NamedAuthProviderClientFactory";
import StitchRedirectCredential from "../providers/StitchRedirectCredential";
import StitchAuth from "../StitchAuth";
import StitchAuthListener from "../StitchAuthListener";
import StitchUser from "../StitchUser";
import StitchBrowserAppAuthRoutes from "./StitchBrowserAppAuthRoutes";
interface PartialLocation {
    hash: string;
    protocol: string;
    host: string;
    pathname: string;
    replace(url: string): any;
}
interface PartialHistory {
    replaceState(data: any, title?: string, url?: string | null): any;
}
interface PartialWindow {
    location: PartialLocation;
    history: PartialHistory;
}
export default class StitchAuthImpl extends CoreStitchAuth<StitchUser> implements StitchAuth {
    private readonly browserAuthRoutes;
    private readonly authStorage;
    private readonly appInfo;
    private readonly jsdomWindow;
    private readonly listeners;
    static injectedFetch?: any;
    constructor(requestClient: StitchRequestClient, browserAuthRoutes: StitchBrowserAppAuthRoutes, authStorage: Storage, appInfo: StitchAppClientInfo, jsdomWindow?: PartialWindow);
    protected readonly userFactory: StitchUserFactory<StitchUser>;
    getProviderClient<ClientT>(factory: AuthProviderClientFactory<ClientT> | NamedAuthProviderClientFactory<ClientT>, providerName?: string): ClientT;
    loginWithCredential(credential: StitchCredential): Promise<StitchUser>;
    loginWithRedirect(credential: StitchRedirectCredential): void;
    linkWithRedirectInternal(user: StitchUser, credential: StitchRedirectCredential): any;
    hasRedirectResult(): boolean;
    handleRedirectResult(): Promise<StitchUser>;
    linkWithCredential(user: CoreStitchUser, credential: StitchCredential): Promise<StitchUser>;
    logout(): Promise<void>;
    protected readonly deviceInfo: {};
    addAuthListener(listener: StitchAuthListener): void;
    removeAuthListener(listener: StitchAuthListener): void;
    onAuthEvent(listener?: StitchAuthListener): void;
    private cleanupRedirect;
    private parseRedirect;
    private processRedirectResult;
    private prepareRedirect;
    private pageRootUrl;
}
export {};
