var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "detect-browser", "mongodb-stitch-core-sdk", "../../internal/common/Version", "./RedirectFragmentFields", "./RedirectKeys", "./StitchRedirectError", "./StitchUserFactoryImpl", "fetch-everywhere"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var detect_browser_1 = require("detect-browser");
    var mongodb_stitch_core_sdk_1 = require("mongodb-stitch-core-sdk");
    var Version_1 = __importDefault(require("../../internal/common/Version"));
    var RedirectFragmentFields_1 = __importDefault(require("./RedirectFragmentFields"));
    var RedirectKeys_1 = __importDefault(require("./RedirectKeys"));
    var StitchRedirectError_1 = __importDefault(require("./StitchRedirectError"));
    var StitchUserFactoryImpl_1 = __importDefault(require("./StitchUserFactoryImpl"));
    var fetch_everywhere_1 = __importDefault(require("fetch-everywhere"));
    var alphaNumericCharacters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    var StitchAuthImpl = (function (_super) {
        __extends(StitchAuthImpl, _super);
        function StitchAuthImpl(requestClient, browserAuthRoutes, authStorage, appInfo, jsdomWindow) {
            if (jsdomWindow === void 0) { jsdomWindow = window; }
            var _this = _super.call(this, requestClient, browserAuthRoutes, authStorage) || this;
            _this.browserAuthRoutes = browserAuthRoutes;
            _this.authStorage = authStorage;
            _this.appInfo = appInfo;
            _this.jsdomWindow = jsdomWindow;
            _this.listeners = new Set();
            return _this;
        }
        Object.defineProperty(StitchAuthImpl.prototype, "userFactory", {
            get: function () {
                return new StitchUserFactoryImpl_1.default(this);
            },
            enumerable: true,
            configurable: true
        });
        StitchAuthImpl.prototype.getProviderClient = function (factory, providerName) {
            if (isAuthProviderClientFactory(factory)) {
                return factory.getClient(this, this.requestClient, this.authRoutes);
            }
            else {
                return factory.getNamedClient(providerName, this.requestClient, this.authRoutes);
            }
        };
        StitchAuthImpl.prototype.loginWithCredential = function (credential) {
            return _super.prototype.loginWithCredentialInternal.call(this, credential);
        };
        StitchAuthImpl.prototype.loginWithRedirect = function (credential) {
            var _a = this.prepareRedirect(credential), redirectUrl = _a.redirectUrl, state = _a.state;
            this.jsdomWindow.location.replace(this.browserAuthRoutes.getAuthProviderRedirectRoute(credential, redirectUrl, state, this.deviceInfo));
        };
        StitchAuthImpl.prototype.linkWithRedirectInternal = function (user, credential) {
            var _this = this;
            if (this.user !== undefined && user.id !== this.user.id) {
                return Promise.reject(new mongodb_stitch_core_sdk_1.StitchClientError(mongodb_stitch_core_sdk_1.StitchClientErrorCode.UserNoLongerValid));
            }
            var _a = this.prepareRedirect(credential), redirectUrl = _a.redirectUrl, state = _a.state;
            var link = this.browserAuthRoutes.getAuthProviderLinkRedirectRoute(credential, redirectUrl, state, this.deviceInfo);
            return (StitchAuthImpl.injectedFetch ? StitchAuthImpl.injectedFetch : fetch_everywhere_1.default)(new Request(link, {
                credentials: "include",
                headers: {
                    Authorization: "Bearer " + this.authInfo.accessToken
                },
                mode: 'cors'
            })).then(function (response) {
                _this.jsdomWindow.location.replace(response.headers.get("X-Stitch-Location"));
            });
        };
        StitchAuthImpl.prototype.hasRedirectResult = function () {
            var isValid = false;
            try {
                isValid = this.parseRedirect().isValid;
                return isValid;
            }
            catch (_) {
                return false;
            }
            finally {
                if (!isValid) {
                    this.cleanupRedirect();
                }
            }
        };
        StitchAuthImpl.prototype.handleRedirectResult = function () {
            try {
                var providerName = this.authStorage.get(RedirectKeys_1.default.ProviderName);
                var providerType = this.authStorage.get(RedirectKeys_1.default.ProviderType);
                var redirectFragment = this.parseRedirect();
                return this.loginWithCredentialInternal(new mongodb_stitch_core_sdk_1.StitchAuthResponseCredential(this.processRedirectResult(redirectFragment), providerType, providerName, redirectFragment.asLink)).then(function (user) {
                    return user;
                });
            }
            catch (err) {
                return Promise.reject(err);
            }
        };
        StitchAuthImpl.prototype.linkWithCredential = function (user, credential) {
            return _super.prototype.linkUserWithCredentialInternal.call(this, user, credential);
        };
        StitchAuthImpl.prototype.logout = function () {
            return Promise.resolve(_super.prototype.logoutInternal.call(this));
        };
        Object.defineProperty(StitchAuthImpl.prototype, "deviceInfo", {
            get: function () {
                var info = {};
                if (this.hasDeviceId) {
                    info[mongodb_stitch_core_sdk_1.DeviceFields.DEVICE_ID] = this.deviceId;
                }
                if (this.appInfo.localAppName !== undefined) {
                    info[mongodb_stitch_core_sdk_1.DeviceFields.APP_ID] = this.appInfo.localAppName;
                }
                if (this.appInfo.localAppVersion !== undefined) {
                    info[mongodb_stitch_core_sdk_1.DeviceFields.APP_VERSION] = this.appInfo.localAppVersion;
                }
                var browser = detect_browser_1.detect();
                if (browser) {
                    info[mongodb_stitch_core_sdk_1.DeviceFields.PLATFORM] = browser.name;
                    info[mongodb_stitch_core_sdk_1.DeviceFields.PLATFORM_VERSION] = browser.version;
                }
                else {
                    info[mongodb_stitch_core_sdk_1.DeviceFields.PLATFORM] = "web";
                    info[mongodb_stitch_core_sdk_1.DeviceFields.PLATFORM_VERSION] = "0.0.0";
                }
                info[mongodb_stitch_core_sdk_1.DeviceFields.SDK_VERSION] = Version_1.default;
                return info;
            },
            enumerable: true,
            configurable: true
        });
        StitchAuthImpl.prototype.addAuthListener = function (listener) {
            this.listeners.add(listener);
            this.onAuthEvent(listener);
        };
        StitchAuthImpl.prototype.removeAuthListener = function (listener) {
            this.listeners.delete(listener);
        };
        StitchAuthImpl.prototype.onAuthEvent = function (listener) {
            var _this = this;
            if (listener) {
                var auth_1 = this;
                var _1 = new Promise(function (resolve) {
                    listener.onAuthEvent(auth_1);
                    resolve(undefined);
                });
            }
            else {
                this.listeners.forEach(function (one) {
                    _this.onAuthEvent(one);
                });
            }
        };
        StitchAuthImpl.prototype.cleanupRedirect = function () {
            this.jsdomWindow.history.replaceState(null, "", this.pageRootUrl());
            this.authStorage.remove(RedirectKeys_1.default.State);
            this.authStorage.remove(RedirectKeys_1.default.ProviderName);
            this.authStorage.remove(RedirectKeys_1.default.ProviderType);
        };
        StitchAuthImpl.prototype.parseRedirect = function () {
            if (typeof this.jsdomWindow === "undefined") {
                throw new StitchRedirectError_1.default("running in a non-browser environment");
            }
            if (!this.jsdomWindow.location || !this.jsdomWindow.location.hash) {
                throw new StitchRedirectError_1.default("window location hash was undefined");
            }
            var ourState = this.authStorage.get(RedirectKeys_1.default.State);
            var redirectFragment = this.jsdomWindow.location.hash.substring(1);
            return parseRedirectFragment(redirectFragment, ourState, this.appInfo.clientAppId);
        };
        StitchAuthImpl.prototype.processRedirectResult = function (redirectFragment) {
            try {
                if (!redirectFragment.isValid) {
                    throw new StitchRedirectError_1.default("invalid redirect result");
                }
                if (redirectFragment.lastError) {
                    throw new StitchRedirectError_1.default("error handling redirect: " + redirectFragment.lastError);
                }
                if (!redirectFragment.authInfo) {
                    throw new StitchRedirectError_1.default("no user auth value was found: it could not be decoded from fragment");
                }
            }
            catch (err) {
                throw err;
            }
            finally {
                this.cleanupRedirect();
            }
            return redirectFragment.authInfo;
        };
        StitchAuthImpl.prototype.prepareRedirect = function (credential) {
            this.authStorage.set(RedirectKeys_1.default.ProviderName, credential.providerName);
            this.authStorage.set(RedirectKeys_1.default.ProviderType, credential.providerType);
            var redirectUrl = credential.redirectUrl;
            if (redirectUrl === undefined) {
                redirectUrl = this.pageRootUrl();
            }
            var state = generateState();
            this.authStorage.set(RedirectKeys_1.default.State, state);
            return { redirectUrl: redirectUrl, state: state };
        };
        StitchAuthImpl.prototype.pageRootUrl = function () {
            return [
                this.jsdomWindow.location.protocol,
                "//",
                this.jsdomWindow.location.host,
                this.jsdomWindow.location.pathname
            ].join("");
        };
        return StitchAuthImpl;
    }(mongodb_stitch_core_sdk_1.CoreStitchAuth));
    exports.default = StitchAuthImpl;
    function generateState() {
        var state = "";
        for (var i = 0; i < 64; ++i) {
            state += alphaNumericCharacters.charAt(Math.floor(Math.random() * alphaNumericCharacters.length));
        }
        return state;
    }
    function unmarshallUserAuth(data) {
        var parts = data.split("$");
        if (parts.length !== 4) {
            throw new StitchRedirectError_1.default("invalid user auth data provided while " +
                "marshalling user authentication data: " +
                data);
        }
        var accessToken = parts[0], refreshToken = parts[1], userId = parts[2], deviceId = parts[3];
        return new mongodb_stitch_core_sdk_1.AuthInfo(userId, deviceId, accessToken, refreshToken);
    }
    var ParsedRedirectFragment = (function () {
        function ParsedRedirectFragment() {
            this.stateValid = false;
            this.clientAppIdValid = false;
            this.asLink = false;
        }
        Object.defineProperty(ParsedRedirectFragment.prototype, "isValid", {
            get: function () {
                return this.stateValid && this.clientAppIdValid;
            },
            enumerable: true,
            configurable: true
        });
        return ParsedRedirectFragment;
    }());
    function parseRedirectFragment(fragment, ourState, ourClientAppId) {
        var vars = fragment.split("&");
        var result = new ParsedRedirectFragment();
        vars.forEach(function (kvp) {
            var pairParts = kvp.split("=");
            var pairKey = decodeURIComponent(pairParts[0]);
            switch (pairKey) {
                case RedirectFragmentFields_1.default.StitchError:
                    result.lastError = decodeURIComponent(pairParts[1]);
                    break;
                case RedirectFragmentFields_1.default.UserAuth:
                    try {
                        result.authInfo = unmarshallUserAuth(decodeURIComponent(pairParts[1]));
                    }
                    catch (e) {
                        result.lastError = e;
                    }
                    break;
                case RedirectFragmentFields_1.default.StitchLink:
                    if (pairParts[1] == "ok") {
                        result.asLink = true;
                    }
                    break;
                case RedirectFragmentFields_1.default.State:
                    var theirState = decodeURIComponent(pairParts[1]);
                    if (ourState === theirState) {
                        result.stateValid = true;
                    }
                    break;
                case RedirectFragmentFields_1.default.ClientAppId:
                    var clientAppId = decodeURIComponent(pairParts[1]);
                    if (ourClientAppId === clientAppId) {
                        result.clientAppIdValid = true;
                    }
                    break;
                default:
                    break;
            }
        });
        return result;
    }
    function isAuthProviderClientFactory(factory) {
        return (factory.getClient !== undefined);
    }
});
//# sourceMappingURL=StitchAuthImpl.js.map